@@ -41,6 +41,7 @@ typedef mbedtls_md_context_t *SHA384CTX;
 typedef mbedtls_md_context_t *SHA512CTX;
 typedef mbedtls_md_context_t *MD5CTX;
 typedef mbedtls_md_context_t *HMACCTX;
+typedef mbedtls_md_context_t *EVPCTX;
 
 #define SHA_DIGEST_LENGTH 20
 #define SHA_DIGEST_LEN SHA_DIGEST_LENGTH
@@ -72,6 +73,10 @@ struct mbedtls_ecdsa_sig {
     bignum s;
 };
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 bignum ssh_mbedcry_bn_new(void);
 void ssh_mbedcry_bn_free(bignum num);
 unsigned char *ssh_mbedcry_bn2num(const_bignum num, int radix);
@@ -135,5 +140,9 @@ ssh_string make_ecpoint_string(const mbedtls_ecp_group *g, const
 
 #define ssh_fips_mode() false
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* HAVE_LIBMBEDCRYPTO */
 #endif /* LIBMBEDCRYPTO_H_ */
