@@ -30,8 +30,12 @@
 
 #include "libssh/config_parser.h"
 #include "libssh/priv.h"
-#include "libssh/misc.h"
 
+/* Returns the original string after skipping the leading whitespace
+ * until finding LF.
+ * This is useful in case we need to get the rest of the line (for example
+ * external command).
+ */
 char *ssh_config_get_cmd(char **str)
 {
     register char *c;
@@ -43,16 +47,7 @@ char *ssh_config_get_cmd(char **str)
             break;
         }
     }
-
-    if (*c == '\"') {
-        for (r = ++c; *c; c++) {
-            if (*c == '\"') {
-                *c = '\0';
-                goto out;
-            }
-        }
-    }
-
+    
     for (r = c; *c; c++) {
         if (*c == '\n') {
             *c = '\0';
@@ -66,23 +61,55 @@ out:
     return r;
 }
 
+/* Returns the next token delimited by whitespace or equal sign (=)
+ * respecting the quotes creating separate token (including whitespaces).
+ */
 char *ssh_config_get_token(char **str)
 {
     register char *c;
-    char *r;
+    bool had_equal = false;
+    char *r = NULL;
 
-    c = ssh_config_get_cmd(str);
-
-    for (r = c; *c; c++) {
-        if (isblank(*c) || *c == '=') {
-            *c = '\0';
-            goto out;
+    /* Ignore leading spaces */
+    for (c = *str; *c; c++) {
+        if (! isblank(*c)) {
+            break;
         }
     }
 
-out:
-    *str = c + 1;
+    /* If we start with quote, return the whole quoted block */
+    if (*c == '\"') {
+        for (r = ++c; *c; c++) {
+            if (*c == '\"' || *c == '\n') {
+                *c = '\0';
+                c++;
+                break;
+            }
+            /* XXX Unmatched quotes extend to the end of line */
+        }
+    } else {
+        /* Otherwise terminate on space, equal or newline */
+        for (r = c; *c; c++) {
+            if (*c == '\0') {
+                goto out;
+            } else if (isblank(*c) || *c == '=' || *c == '\n') {
+                had_equal = (*c == '=');
+                *c = '\0';
+                c++;
+                break;
+            }
+        }
+    }
 
+    /* Skip any other remaining whitespace */
+    while (isblank(*c) || *c == '\n' || (!had_equal && *c == '=')) {
+        if (*c == '=') {
+            had_equal = true;
+        }
+        c++;
+    }
+out:
+    *str = c;
     return r;
 }
 
@@ -140,7 +167,6 @@ int ssh_config_parse_uri(const char *tok,
 {
     char *endp = NULL;
     long port_n;
-    int rc;
 
     /* Sanitize inputs */
     if (username != NULL) {
@@ -154,7 +180,7 @@ int ssh_config_parse_uri(const char *tok,
     }
 
     /* Username part (optional) */
-    endp = strrchr(tok, '@');
+    endp = strchr(tok, '@');
     if (endp != NULL) {
         /* Zero-length username is not valid */
         if (tok == endp) {
@@ -198,14 +224,6 @@ int ssh_config_parse_uri(const char *tok,
         if (*hostname == NULL) {
             goto error;
         }
-        /* if not an ip, check syntax */
-        rc = ssh_is_ipaddr(*hostname);
-        if (rc == 0) {
-            rc = ssh_check_hostname_syntax(*hostname);
-            if (rc != SSH_OK) {
-                goto error;
-            }
-        }
     }
     /* Skip also the closing bracket */
     if (*endp == ']') {
