@@ -136,7 +136,7 @@ static int getai(const char *host, int port, struct addrinfo **ai)
 #endif
     }
 
-    if (ssh_is_ipaddr(host) == 1) {
+    if (ssh_is_ipaddr(host)) {
         /* this is an IP address */
         SSH_LOG(SSH_LOG_PACKET, "host %s matches an IP address", host);
         hints.ai_flags |= AI_NUMERICHOST;
@@ -183,11 +183,13 @@ socket_t ssh_connect_host_nonblocking(ssh_session session, const char *host,
     }
 
     for (itr = ai; itr != NULL; itr = itr->ai_next) {
+        char err_msg[SSH_ERRNO_MSG_MAX] = {0};
         /* create socket */
         s = socket(itr->ai_family, itr->ai_socktype, itr->ai_protocol);
         if (s < 0) {
             ssh_set_error(session, SSH_FATAL,
-                          "Socket create failed: %s", strerror(errno));
+                          "Socket create failed: %s",
+                          ssh_strerror(errno, err_msg, SSH_ERRNO_MSG_MAX));
             continue;
         }
 
@@ -214,7 +216,8 @@ socket_t ssh_connect_host_nonblocking(ssh_session session, const char *host,
             {
                 if (bind(s, bind_itr->ai_addr, bind_itr->ai_addrlen) < 0) {
                     ssh_set_error(session, SSH_FATAL,
-                                  "Binding local address: %s", strerror(errno));
+                                  "Binding local address: %s",
+                                  ssh_strerror(errno, err_msg, SSH_ERRNO_MSG_MAX));
                     continue;
                 } else {
                     break;
@@ -246,7 +249,7 @@ socket_t ssh_connect_host_nonblocking(ssh_session session, const char *host,
             if (rc < 0) {
                 ssh_set_error(session, SSH_FATAL,
                               "Failed to set TCP_NODELAY on socket: %s",
-                              strerror(errno));
+                              ssh_strerror(errno, err_msg, SSH_ERRNO_MSG_MAX));
                 ssh_connect_socket_close(s);
                 s = -1;
                 continue;
@@ -257,7 +260,8 @@ socket_t ssh_connect_host_nonblocking(ssh_session session, const char *host,
         rc = connect(s, itr->ai_addr, itr->ai_addrlen);
         if (rc == -1 && (errno != 0) && (errno != EINPROGRESS)) {
             ssh_set_error(session, SSH_FATAL,
-                          "Failed to connect: %s", strerror(errno));
+                          "Failed to connect: %s",
+                          ssh_strerror(errno, err_msg, SSH_ERRNO_MSG_MAX));
             ssh_connect_socket_close(s);
             s = -1;
             continue;
@@ -289,14 +293,14 @@ static int ssh_select_cb (socket_t fd, int revents, void *userdata)
 /**
  * @brief A wrapper for the select syscall
  *
- * This functions acts more or less like the select(2) syscall.\n
+ * This function acts more or less like the select(2) syscall.\n
  * There is no support for writing or exceptions.\n
  *
  * @param[in]  channels Arrays of channels pointers terminated by a NULL.
  *                      It is never rewritten.
  *
- * @param[out] outchannels Arrays of same size that "channels", there is no need
- *                         to initialize it.
+ * @param[out] outchannels Arrays of the same size as "channels", there is no
+ *                         need to initialize it.
  *
  * @param[in]  maxfd    Maximum +1 file descriptor from readfds.
  *
