@@ -366,6 +366,11 @@ static enum ssh_packet_filter_result_e ssh_packet_incoming_filter(ssh_session se
          * - session->dh_handshake_state = DH_STATE_NEWKEYS_SENT
          * */
 
+        if (!session->server) {
+            rc = SSH_PACKET_DENIED;
+            break;
+        }
+
         if (session->session_state != SSH_SESSION_STATE_DH) {
             rc = SSH_PACKET_DENIED;
             break;
@@ -1052,9 +1057,11 @@ static bool ssh_packet_need_rekey(ssh_session session,
  * @param user pointer to current ssh_session
  * @param data pointer to the data received
  * @len length of data received. It might not be enough for a complete packet
- * @returns number of bytes read and processed.
+ * @returns number of bytes read and processed. Zero means only partial packet
+ * received and negative value means error.
  */
-size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *user)
+size_t
+ssh_packet_socket_callback(const void *data, size_t receivedlen, void *user)
 {
     ssh_session session = (ssh_session)user;
     uint32_t blocksize = 8;
@@ -1066,7 +1073,7 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
     uint8_t *cleartext_packet = NULL;
     uint8_t *packet_second_block = NULL;
     uint8_t *mac = NULL;
-    size_t packet_remaining;
+    size_t packet_remaining, packet_offset;
     uint32_t packet_len, compsize, payloadsize;
     uint8_t padding;
     size_t processed = 0; /* number of bytes processed from the callback */
@@ -1112,7 +1119,7 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
                     session->packet_state == PACKET_STATE_PROCESSING ?
                     "PROCESSING" : "unknown");
 #endif
-    switch(session->packet_state) {
+    switch (session->packet_state) {
         case PACKET_STATE_INIT:
             if (receivedlen < lenfield_blocksize + etm_packet_offset) {
                 /*
@@ -1145,11 +1152,13 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
             }
 
             if (!etm) {
-                ptr = ssh_buffer_allocate(session->in_buffer, lenfield_blocksize);
+                ptr = ssh_buffer_allocate(session->in_buffer,
+                                          lenfield_blocksize);
                 if (ptr == NULL) {
                     goto error;
                 }
-                packet_len = ssh_packet_decrypt_len(session, ptr, (uint8_t *)data);
+                packet_len = ssh_packet_decrypt_len(session, ptr,
+                                                    (uint8_t *)data);
                 to_be_read = packet_len - lenfield_blocksize + sizeof(uint32_t);
             } else {
                 /* Length is unencrypted in case of Encrypt-then-MAC */
@@ -1179,11 +1188,11 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
             FALL_THROUGH;
         case PACKET_STATE_SIZEREAD:
             packet_len = session->in_packet.len;
-            processed = lenfield_blocksize + etm_packet_offset;
+            packet_offset = processed = lenfield_blocksize + etm_packet_offset;
             to_be_read = packet_len + sizeof(uint32_t) + current_macsize;
             /* if to_be_read is zero, the whole packet was blocksize bytes. */
             if (to_be_read != 0) {
-                if (receivedlen  < (unsigned long)to_be_read) {
+                if (receivedlen < (unsigned long)to_be_read) {
                     /* give up, not enough data in buffer */
                     SSH_LOG(SSH_LOG_PACKET,
                             "packet: partial packet (read len) "
@@ -1194,13 +1203,12 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
                     return 0;
                 }
 
-                packet_second_block = (uint8_t*)data + lenfield_blocksize + etm_packet_offset;
+                packet_second_block = (uint8_t*)data + packet_offset;
                 processed = to_be_read - current_macsize;
             }
 
             /* remaining encrypted bytes from the packet, MAC not included */
-            packet_remaining =
-                packet_len - (lenfield_blocksize - sizeof(uint32_t) + etm_packet_offset);
+            packet_remaining = packet_len - (packet_offset - sizeof(uint32_t));
             cleartext_packet = ssh_buffer_allocate(session->in_buffer,
                                                    packet_remaining);
             if (cleartext_packet == NULL) {
@@ -1223,16 +1231,16 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
                         }
                     }
                     /*
-                     * Decrypt the packet. In case of EtM mode, the length is already
-                     * known as it's unencrypted. In the other case, lenfield_blocksize bytes
-                     * already have been decrypted.
+                     * Decrypt the packet. In case of EtM mode, the length is
+                     * already known as it's unencrypted. In the other case,
+                     * lenfield_blocksize bytes already have been decrypted.
                      */
                     if (packet_remaining > 0) {
                         rc = ssh_packet_decrypt(session,
                                                 cleartext_packet,
                                                 (uint8_t *)data,
-                                                lenfield_blocksize + etm_packet_offset,
-                                                processed - (lenfield_blocksize + etm_packet_offset));
+                                                packet_offset,
+                                                processed - packet_offset);
                         if (rc < 0) {
                             ssh_set_error(session,
                                           SSH_FATAL,
@@ -1242,9 +1250,10 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
                     }
 
                     if (crypto->in_hmac != SSH_HMAC_NONE && !etm) {
+                        ssh_buffer in = session->in_buffer;
                         rc = ssh_packet_hmac_verify(session,
-                                                    ssh_buffer_get(session->in_buffer),
-                                                    ssh_buffer_get_len(session->in_buffer),
+                                                    ssh_buffer_get(in),
+                                                    ssh_buffer_get_len(in),
                                                     mac,
                                                     crypto->in_hmac);
                         if (rc < 0) {
@@ -1295,9 +1304,10 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
             compsize = ssh_buffer_get_len(session->in_buffer);
 
 #ifdef WITH_ZLIB
-            if (crypto && crypto->do_compress_in
-                && ssh_buffer_get_len(session->in_buffer) > 0) {
-                rc = decompress_buffer(session, session->in_buffer,MAX_PACKET_LEN);
+            if (crypto && crypto->do_compress_in &&
+                ssh_buffer_get_len(session->in_buffer) > 0) {
+                rc = decompress_buffer(session, session->in_buffer,
+                                       MAX_PACKET_LEN);
                 if (rc < 0) {
                     goto error;
                 }
@@ -1324,13 +1334,15 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
             session->packet_state = PACKET_STATE_PROCESSING;
             ssh_packet_parse_type(session);
             SSH_LOG(SSH_LOG_PACKET,
-                    "packet: read type %hhd [len=%d,padding=%hhd,comp=%d,payload=%d]",
-                    session->in_packet.type, packet_len, padding, compsize, payloadsize);
+                    "packet: read type %hhd [len=%" PRIu32 ",padding=%hhd,"
+                    "comp=%" PRIu32 ",payload=%" PRIu32 "]",
+                    session->in_packet.type, packet_len, padding, compsize,
+                    payloadsize);
 
             /* Check if the packet is expected */
             filter_result = ssh_packet_incoming_filter(session);
 
-            switch(filter_result) {
+            switch (filter_result) {
             case SSH_PACKET_ALLOWED:
                 /* Execute callbacks */
                 ssh_packet_process(session, session->in_packet.type);
@@ -1355,7 +1367,8 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
 
                 ptr = ((uint8_t*)data) + processed;
 
-                rc = ssh_packet_socket_callback(ptr, receivedlen - processed,user);
+                rc = ssh_packet_socket_callback(ptr, receivedlen - processed,
+                                                user);
                 processed += rc;
             }
 
@@ -1381,8 +1394,8 @@ size_t ssh_packet_socket_callback(const void *data, size_t receivedlen, void *us
                   session->packet_state);
 
 error:
-    session->session_state= SSH_SESSION_STATE_ERROR;
-    SSH_LOG(SSH_LOG_PACKET,"Packet: processed %zu bytes", processed);
+    session->session_state = SSH_SESSION_STATE_ERROR;
+    SSH_LOG(SSH_LOG_PACKET, "Packet: processed %zu bytes", processed);
     return processed;
 }
 
@@ -1410,18 +1423,23 @@ static void ssh_packet_socket_controlflow_callback(int code, void *userdata)
     }
 }
 
-void ssh_packet_register_socket_callback(ssh_session session, ssh_socket s){
-	session->socket_callbacks.data=ssh_packet_socket_callback;
-	session->socket_callbacks.connected=NULL;
-    session->socket_callbacks.controlflow = ssh_packet_socket_controlflow_callback;
-	session->socket_callbacks.userdata=session;
-	ssh_socket_set_callbacks(s,&session->socket_callbacks);
+void ssh_packet_register_socket_callback(ssh_session session, ssh_socket s)
+{
+    struct ssh_socket_callbacks_struct *callbacks = &session->socket_callbacks;
+
+    callbacks->data = ssh_packet_socket_callback;
+    callbacks->connected = NULL;
+    callbacks->controlflow = ssh_packet_socket_controlflow_callback;
+    callbacks->userdata = session;
+    ssh_socket_set_callbacks(s, callbacks);
 }
 
 /** @internal
  * @brief sets the callbacks for the packet layer
  */
-void ssh_packet_set_callbacks(ssh_session session, ssh_packet_callbacks callbacks){
+void
+ssh_packet_set_callbacks(ssh_session session, ssh_packet_callbacks callbacks)
+{
     if (session->packet_callbacks == NULL) {
         session->packet_callbacks = ssh_list_new();
         if (session->packet_callbacks == NULL) {
@@ -1435,8 +1453,11 @@ void ssh_packet_set_callbacks(ssh_session session, ssh_packet_callbacks callback
 /** @internal
  * @brief remove the callbacks from the packet layer
  */
-void ssh_packet_remove_callbacks(ssh_session session, ssh_packet_callbacks callbacks){
+void
+ssh_packet_remove_callbacks(ssh_session session, ssh_packet_callbacks callbacks)
+{
     struct ssh_iterator *it = NULL;
+
     it = ssh_list_find(session->packet_callbacks, callbacks);
     if (it != NULL) {
         ssh_list_remove(session->packet_callbacks, it);
@@ -1446,12 +1467,15 @@ void ssh_packet_remove_callbacks(ssh_session session, ssh_packet_callbacks callb
 /** @internal
  * @brief sets the default packet handlers
  */
-void ssh_packet_set_default_callbacks(ssh_session session){
-	session->default_packet_callbacks.start=1;
-	session->default_packet_callbacks.n_callbacks=sizeof(default_packet_handlers)/sizeof(ssh_packet_callback);
-	session->default_packet_callbacks.user=session;
-	session->default_packet_callbacks.callbacks=default_packet_handlers;
-	ssh_packet_set_callbacks(session, &session->default_packet_callbacks);
+void ssh_packet_set_default_callbacks(ssh_session session)
+{
+    struct ssh_packet_callbacks_struct *c = &session->default_packet_callbacks;
+
+    c->start = 1;
+    c->n_callbacks = sizeof(default_packet_handlers) / sizeof(ssh_packet_callback);
+    c->user = session;
+    c->callbacks = default_packet_handlers;
+    ssh_packet_set_callbacks(session, c);
 }
 
 /** @internal
@@ -1755,10 +1779,12 @@ static bool
 ssh_packet_in_rekey(ssh_session session)
 {
     /* We know we are rekeying if we are authenticated and the DH
-     * status is not finished
+     * status is not finished, but we only queue packets until we've
+     * sent our NEWKEYS.
      */
     return (session->flags & SSH_SESSION_FLAG_AUTHENTICATED) &&
-           (session->dh_handshake_state != DH_STATE_FINISHED);
+           (session->dh_handshake_state != DH_STATE_FINISHED) &&
+           (session->dh_handshake_state != DH_STATE_NEWKEYS_SENT);
 }
 
 int ssh_packet_send(ssh_session session)
@@ -1799,7 +1825,7 @@ int ssh_packet_send(ssh_session session)
 
         if (need_rekey) {
             /* Send the KEXINIT packet instead.
-             * This recursivelly calls the packet_send(), but it should
+             * This recursively calls the packet_send(), but it should
              * not get into rekeying again.
              * After that we need to handle the key exchange responses
              * up to the point where we can send the rest of the queue.
@@ -1930,7 +1956,7 @@ ssh_packet_set_newkeys(ssh_session session,
         memcpy(session->next_crypto->session_id,
                session->current_crypto->session_id,
                session_id_len);
-	session->next_crypto->session_id_len = session_id_len;
+        session->next_crypto->session_id_len = session_id_len;
 
         return SSH_OK;
     }
