@@ -44,7 +44,7 @@
 #endif
 
 /**
- * @defgroup libssh_poll The SSH poll functions.
+ * @defgroup libssh_poll The SSH poll functions
  * @ingroup libssh
  *
  * Add a generic way to handle sockets asynchronously.
@@ -68,7 +68,7 @@ struct ssh_poll_handle_struct {
     size_t idx;
   } x;
   short events;
-  int lock;
+  uint32_t lock_cnt;
   ssh_poll_callback cb;
   void *cb_data;
 };
@@ -287,7 +287,7 @@ static int bsd_poll(ssh_pollfd_t *fds, nfds_t nfds, int timeout)
     if (rc < 0) {
         return -1;
     }
-    /* A timeout occured */
+    /* A timeout occurred */
     if (rc == 0) {
         return 0;
     }
@@ -412,7 +412,8 @@ short ssh_poll_get_events(ssh_poll_handle p)
 
 /**
  * @brief  Set the events of a poll object. The events will also be propagated
- *         to an associated poll context.
+ *         to an associated poll context unless the fd is locked. In that case,
+ *         only the POLLOUT can be set.
  *
  * @param  p            Pointer to an already allocated poll object.
  * @param  events       Poll events.
@@ -420,8 +421,14 @@ short ssh_poll_get_events(ssh_poll_handle p)
 void ssh_poll_set_events(ssh_poll_handle p, short events)
 {
     p->events = events;
-    if (p->ctx != NULL && !p->lock) {
-        p->ctx->pollfds[p->x.idx].events = events;
+    if (p->ctx != NULL) {
+        if (p->lock_cnt == 0) {
+            p->ctx->pollfds[p->x.idx].events = events;
+        } else if (!(p->ctx->pollfds[p->x.idx].events & POLLOUT)) {
+            /* if locked, allow only setting POLLOUT to prevent recursive
+             * callbacks */
+            p->ctx->pollfds[p->x.idx].events = events & POLLOUT;
+        }
     }
 }
 
@@ -621,7 +628,7 @@ int ssh_poll_ctx_add(ssh_poll_ctx ctx, ssh_poll_handle p)
  */
 int ssh_poll_ctx_add_socket (ssh_poll_ctx ctx, ssh_socket s)
 {
-    ssh_poll_handle p;
+    ssh_poll_handle p = NULL;
     int ret;
 
     p = ssh_socket_get_poll_handle(s);
@@ -691,6 +698,15 @@ int ssh_poll_ctx_dopoll(ssh_poll_ctx ctx, int timeout)
         return SSH_ERROR;
     }
 
+    /* Ignore any pollin events on locked sockets as that means we are called
+     * recursively and we only want process the POLLOUT events here to flush
+     * output buffer */
+    for (i = 0; i < ctx->polls_used; i++) {
+        /* The lock prevents invoking POLLIN events: drop them now */
+        if (ctx->pollptrs[i]->lock_cnt > 0) {
+            ctx->pollfds[i].events &= ~POLLIN;
+        }
+    }
     ssh_timestamp_init(&ts);
     do {
         int tm = ssh_timeout_update(&ts, timeout);
@@ -706,7 +722,7 @@ int ssh_poll_ctx_dopoll(ssh_poll_ctx ctx, int timeout)
 
     used = ctx->polls_used;
     for (i = 0; i < used && rc > 0; ) {
-        if (!ctx->pollfds[i].revents || ctx->pollptrs[i]->lock) {
+        if (ctx->pollfds[i].revents == 0) {
             i++;
         } else {
             int ret;
@@ -716,7 +732,7 @@ int ssh_poll_ctx_dopoll(ssh_poll_ctx ctx, int timeout)
             revents = ctx->pollfds[i].revents;
             /* avoid having any event caught during callback */
             ctx->pollfds[i].events = 0;
-            p->lock = 1;
+            p->lock_cnt++;
             if (p->cb && (ret = p->cb(p, fd, revents, p->cb_data)) < 0) {
                 if (ret == -2) {
                     return -1;
@@ -727,7 +743,7 @@ int ssh_poll_ctx_dopoll(ssh_poll_ctx ctx, int timeout)
             } else {
                 ctx->pollfds[i].revents = 0;
                 ctx->pollfds[i].events = p->events;
-                p->lock = 0;
+                p->lock_cnt--;
                 i++;
             }
 
@@ -981,7 +997,7 @@ int ssh_event_add_connector(ssh_event event, ssh_connector connector)
  * @returns SSH_OK      on success.
  *          SSH_ERROR   Error happened during the poll. Check errno to get more
  *                      details about why it failed.
- *          SSH_AGAIN   Timeout occured
+ *          SSH_AGAIN   Timeout occurred
  */
 int ssh_event_dopoll(ssh_event event, int timeout)
 {
